@@ -1,16 +1,16 @@
 ''' These classes model the earth's gravity and perform some coordinate transformations between the global inertial frame and the ENU/launch tower frame '''
 
 from abc import ABC, abstractmethod
-from math import atan2, cos, degrees, radians, sin, sqrt, pi
+from math import atan2, cos, degrees, pi, radians, sin, sqrt
 from typing import Union
 
 import numpy as np
 
-from MAPLEAF.Motion.CythonQuaternion import Quaternion
-from MAPLEAF.Motion.CythonVector import Vector
-from MAPLEAF.Motion.Inertia import Inertia
-from MAPLEAF.Motion.RigidBodyStates import RigidBodyState, RigidBodyState_3DoF
-from MAPLEAF.IO.SimDefinition import getAbsoluteFilePath
+from MAPLEAF.IO import getAbsoluteFilePath
+from MAPLEAF.Motion import (Inertia, Quaternion, RigidBodyState,
+                            RigidBodyState_3DoF, Vector)
+
+__all__ = [ "earthModelFactory", "NoEarth", "FlatEarth", "SphericalEarth", "WGS84" ]
 
 class EarthModel(ABC):
     ''' Interface for all earth models '''
@@ -55,7 +55,7 @@ class EarthModel(ABC):
         return
 
 def earthModelFactory(envDictReader=None) -> EarthModel:
-    ''' Provide an envDictReader (`MAPLEAF.IO.SubDictReader.SubDictReader`). If none is provided, returns a FlatEarth model '''
+    ''' Provide an envDictReader (`MAPLEAF.IO.SubDictReader`). If none is provided, returns a FlatEarth model '''
     if envDictReader == None:
         return FlatEarth()
 
@@ -66,7 +66,7 @@ def earthModelFactory(envDictReader=None) -> EarthModel:
     elif earthModel == "Round":
         return SphericalEarth()
     elif earthModel == "WGS84":
-        return WGS84Earth()
+        return WGS84()
     elif earthModel == "None":
         return NoEarth()
     else:
@@ -102,7 +102,7 @@ class FlatEarth(NoEarth):
         gravityMagnitude = inertia.mass * self.GM / distanceFromCenterOfEarth**2 # Checked that this equation gives results similar to the 
             # US Standard Atmosphere - offset a bit depending on what earth radius is used. Current 6371 km gives slightly lower values that USTDA.
 
-        return gravityMagnitude * gravityDirection
+        return gravityDirection * gravityMagnitude
 
 class SphericalEarth(EarthModel):
     ''' Models a non-rotating, uniform, spherical earth '''
@@ -163,7 +163,7 @@ class SphericalEarth(EarthModel):
         gravityMagnitude = inertia.mass * self.GM / distanceFromCenterOfEarth**2 # Checked that this equation gives results similar to the 
             # US Standard Atmosphere - offset a bit depending on what earth radius is used. Current 6371 km gives slightly lower values that USTDA.
 
-        return gravityMagnitude * gravityDirection
+        return gravityDirection * gravityMagnitude
 
     def getInertialToENUFrameRotation(self, x, y, z):
         ''' Returns a Quaternion that defines the rotation b/w the global inertia frame and the local, surface-normal North-East-Up (y-x-z) frame '''
@@ -226,7 +226,7 @@ class SphericalEarth(EarthModel):
         except AttributeError: # 3DoF Case
             return RigidBodyState_3DoF(globalFramePosition, globalFrameVelocity)             
 
-class WGS84Earth(SphericalEarth):
+class WGS84(SphericalEarth):
     ''' 
         Models a rotating, ellipsoid earth, with non-uniform gravity 
         Inherits the getInertialToENUFrameRotation function from SphericalEarth, otherwise overrides everything
@@ -364,7 +364,7 @@ class WGS84Earth(SphericalEarth):
         multiplier = -mu*inertia.mass / r**3
 
         xForce = x * multiplier * (1 - frac*(5*z*z - r*r))
-        yForce = xForce * y/x
+        yForce = y * multiplier * (1 - frac*(5*z*z - r*r))
         zForce = z * multiplier * (1 - frac*(5*z*z - 3*r*r))
 
-        return Vector(xForce, yForce, zForce)
\ No newline at end of file
+        return Vector(xForce, yForce, zForce)
