@@ -6,10 +6,10 @@ import abc
 from math import e
 
 import numpy as np
-from scipy.interpolate import LinearNDInterpolator
 
-import MAPLEAF.Rocket.AeroFunctions as AeroFunctions
+from MAPLEAF.Motion import AeroParameters, NoNaNLinearNDInterpolator
 
+__all__ = [ "TableInterpolatingActuatorController", "FirstOrderActuator", "FirstOrderSystem", "ActuatorController", "Actuator" ]
 
 class FirstOrderSystem():
     def __init__(self, responseTime):
@@ -47,6 +47,7 @@ class TableInterpolatingActuatorController(ActuatorController):
     def __init__(self, deflectionTableFilePath, nKeyColumns, keyFunctionList, actuatorList):
         self.actuatorList = actuatorList
         self.keyFunctionList = keyFunctionList
+        self.deflectionTablePath = deflectionTableFilePath
 
         # Load actuator-deflection data table
         deflData = np.loadtxt(deflectionTableFilePath, skiprows=1)
@@ -60,7 +61,7 @@ class TableInterpolatingActuatorController(ActuatorController):
             raise ValueError("Number of actuators: {}, must match number of actuator deflection columns in deflection table: {}".format(nActuators, nDeflectionTableEntries))
 
         # Create interpolation function for fin deflections
-        self._getPositionTargets = LinearNDInterpolator(keys, deflData)
+        self._getPositionTargets = NoNaNLinearNDInterpolator(keys, deflData)
 
     def setTargetActuatorDeflections(self, desiredMoments, state, environment, time):
         '''
@@ -69,7 +70,7 @@ class TableInterpolatingActuatorController(ActuatorController):
                 time:           time at which the moments are requested (time of current control loop execution)
         '''
         # Construct key vector, starting with non-moment components:
-        keyVector = AeroFunctions.getAeroPropertiesList(self.keyFunctionList, state, environment)
+        keyVector = AeroParameters.getAeroPropertiesList(self.keyFunctionList, state, environment)
         for desiredMoment in desiredMoments:
             keyVector.append(desiredMoment)
 
@@ -80,9 +81,7 @@ class TableInterpolatingActuatorController(ActuatorController):
 
         return list(newActuatorPositionTargets)
 
-
-
-
+    
 class Actuator(abc.ABC):
     ''' 
         Interface for actuators.
