@@ -6,25 +6,26 @@ import abc
 
 import numpy as np
 
-import MAPLEAF.Rocket.AeroFunctions as AeroFunctions
-from MAPLEAF.GNC.PID import GainScheduledPIDController
+from MAPLEAF.Motion import AeroParameters, AngularVelocity, Vector
+from MAPLEAF.GNC import ConstantGainPIDController, ScheduledGainPIDController
 
+__all__ = ["ConstantGainPIDRocketMomentController", "ScheduledGainPIDRocketMomentController", "MomentController", "IdealMomentController" ]
 
 class MomentController(abc.ABC):
     @abc.abstractmethod
     def getDesiredMoments(self, rocketState, environment, targetOrientation, time, dt):
         ''' Should return a list [ desired x-axis, y-axis, and z-axis ] moments '''
 
-class GainScheduledPIDRocketMomentController(MomentController, GainScheduledPIDController):
+class ScheduledGainPIDRocketMomentController(MomentController, ScheduledGainPIDController):
     def __init__(self, gainTableFilePath, keyColumnNames):
         '''
             Assumes the longitudinal (Pitch/Yaw) PID coefficients are in columns nKeyColumns:nKeyColumns+2
             Assumes the roll PID coefficients are in columns nKeyColumns+3:nKeyColumns+5
             Sample gain file: MAPLEAF/Examples/TabulatedData/testPIDCoeffs.txt
         '''
-        self.keyFunctionList = [ AeroFunctions.stringToAeroFunctionMap[x] for x in keyColumnNames ]
+        self.keyFunctionList = [ AeroParameters.stringToAeroFunctionMap[x] for x in keyColumnNames ]
         nKeyColumns = len(keyColumnNames)
-        GainScheduledPIDController.__init__(self, gainTableFilePath, nKeyColumns, PCol=nKeyColumns, DCol=nKeyColumns+5)
+        ScheduledGainPIDController.__init__(self, gainTableFilePath, nKeyColumns, PCol=nKeyColumns, DCol=nKeyColumns+5)
         
     def updateCoefficientsFromGainTable(self, keyList):
         ''' Overriding parent class method to enable separate longitudinal and roll coefficients in a single controller '''
@@ -50,6 +51,42 @@ class GainScheduledPIDRocketMomentController(MomentController, GainScheduledPIDC
                 dt:                 (numeric) time since last execution of the control system
         '''
         orientationError = self._getOrientationError(rocketState, targetOrientation)
-        gainKeyList = AeroFunctions.getAeroPropertiesList(self.keyFunctionList, rocketState, environment)
+        gainKeyList = AeroParameters.getAeroPropertiesList(self.keyFunctionList, rocketState, environment)
         self.updateCoefficientsFromGainTable(gainKeyList)
         return self.getNewSetPoint(orientationError, dt)
+
+class ConstantGainPIDRocketMomentController(MomentController, ConstantGainPIDController):
+    def __init__(self, Pxy, Ixy, Dxy, Pz, Iz, Dz):
+        '''
+            Constant PID coefficient controller
+        '''
+        ConstantGainPIDController.__init__(self)
+        
+        P = np.array([Pxy, Pxy, Pz])
+        I = np.array([Ixy, Ixy, Iz])
+        D = np.array([Dxy, Dxy, Dz])
+        self.updateCoefficients(P,I,D)
+
+    def _getOrientationError(self, rocketState, targetOrientation):
+        return np.array((targetOrientation / rocketState.orientation).toRotationVector())
+
+    def getDesiredMoments(self, rocketState, environment, targetOrientation, time, dt):
+        '''
+            Inputs: 
+                gainKeyList:        iterable of length nKeyColumns, containing the data required to interpolate the PIDxy and PIDz coefficients in the gain table
+                rocketState:        must have .orientation attribute (Quaternion)
+                targetOrientation:  (Quaternion)
+                _:                  currently unused (time argument)
+                dt:                 (numeric) time since last execution of the control system
+        '''
+        orientationError = self._getOrientationError(rocketState, targetOrientation)
+        return self.getNewSetPoint(orientationError, dt)
+
+class IdealMomentController():
+    def __init__(self, rocket):
+        self.rocket = rocket
+
+    def getDesiredMoments(self, rocketState, environment, targetOrientation, time, dt):
+        """ Exactly match the desired target orientation and set the rockets angular velocity to zero """
+        self.rocket.rigidBody.state.orientation = targetOrientation
+        self.rocket.rigidBody.state.angularVelocity = AngularVelocity(rotationVector=Vector(0,0,0))
\ No newline at end of file
