@@ -2,9 +2,9 @@
 
 import abc
 
-from MAPLEAF.Motion.CythonQuaternion import Quaternion
-from MAPLEAF.Motion.CythonVector import Vector
+from MAPLEAF.Motion import Quaternion, Vector
 
+__all__ = [ "Navigator", "Stabilizer" ]
 
 class Navigator(abc.ABC):
     @abc.abstractmethod
