@@ -1,8 +1,9 @@
 ''' PID controllers control parts of the control system and adaptive simulation timestepping '''
 
 import numpy as np
-from scipy.interpolate import LinearNDInterpolator
+from MAPLEAF.Motion import NoNaNLinearNDInterpolator
 
+__all__ = [ "PIDController", "ConstantGainPIDController", "ScheduledGainPIDController" ]
 
 class PIDController():
 
@@ -24,6 +25,10 @@ class PIDController():
             self.maxIntegralMagnitude = maxIntegral
 
     def getNewSetPoint(self, currentError, dt):
+        # Calculate derivative
+        derivative = (currentError - self.lastError) / dt
+        
+        # Calculate integral term
         self.errorIntegral = self.errorIntegral + (currentError + self.lastError)*dt / 2
 
         # Cap the magnitude of the integral term if necessary
@@ -32,7 +37,7 @@ class PIDController():
                 # Check if vector valued
                 testIter = iter(self.errorIntegral)
                 for i in range(len(self.errorIntegral)):
-                    if abs(self.errorIntegral[i] > self.maxIntegralMagnitude[i]):
+                    if abs(self.errorIntegral[i]) > self.maxIntegralMagnitude[i]:
                         self.errorIntegral[i] = self.errorIntegral[i] * self.maxIntegralMagnitude[i] / abs(self.errorIntegral[i])
 
             except TypeError:
@@ -40,9 +45,9 @@ class PIDController():
                 if abs(self.errorIntegral) > self.maxIntegralMagnitude:
                     self.errorIntegral = self.errorIntegral * self.maxIntegralMagnitude / abs(self.errorIntegral)
 
-        derivative = (currentError - self.lastError) / dt
-
+        # Store error for next calculation
         self.lastError = currentError
+
         return self.P*currentError + self.I*self.errorIntegral + self.D*derivative
 
     def updateCoefficients(self, P, I, D, maxIntegral=None):
@@ -54,7 +59,7 @@ class PIDController():
     def resetIntegral(self):
         self.errorIntegral = self.lastError * 0 # Done to handle arbitrary size np arrays
 
-class GainScheduledPIDController(PIDController):
+class ScheduledGainPIDController(PIDController):
     def __init__(self, gainTableFilePath, nKeyColumns=2, PCol=3, DCol=5, initialError=0, maxIntegral=None):
         '''
             Inputs:
@@ -77,8 +82,26 @@ class GainScheduledPIDController(PIDController):
         pidData = pidData[:,PCol:DCol+1]
 
         #Create interpolation function for PID coefficients
-        self._getPIDCoeffs = LinearNDInterpolator(keys, pidData)
+        self._getPIDCoeffs = NoNaNLinearNDInterpolator(keys, pidData)
 
     def updateCoefficientsFromGainTable(self, keyList):
         P, I, D = self._getPIDCoeffs(keyList)
         self.updateCoefficients(P, I, D)
+
+class ConstantGainPIDController(PIDController):
+
+    def __init__(self, P=0, I=0, D=0, initialError=0, maxIntegral=None):
+        '''
+            Inputs:
+                P:                  (int) Proportional Gain
+                I:                  (int) Integral Gain
+                D:                  (int) Derivative Gain
+                DCol:               (int) zero-indexed column number of D Coefficient
+
+                Note:
+                    It is assumed that PCol, ICol, and DCol exist one after another in the table
+                
+                Inputs passed through to parent class (PICController):
+                    initialError, maxIntegral
+        '''
+        PIDController.__init__(self, P,I,D, initialError=initialError, maxIntegral=maxIntegral)
\ No newline at end of file
