@@ -1,10 +1,13 @@
 '''
-Classes and functions for creating simulation logs for regular simulations (Logger) and Monte Carlo simulations (MonteCarloLogger)
+    Classes and functions for creating simulation logs for regular simulations (Logger) and Monte Carlo simulations (MonteCarloLogger)
 '''
 
 import os
 import sys
 
+from functools import lru_cache
+import MAPLEAF
+
 # TODO: When logging, keep track of messages containing 'error' or 'warning' -> reprint those at the end of the simulation?
 
 class Logger():
@@ -93,6 +96,8 @@ def findNextAvailableNumberedFileName(fileBaseName="monteCarloLog", extension=".
         If fileBaseName is simLog, returns the first of: simLog1, simLog2, simLog3, etc... that isn't already a file.
         Returns a string of the form fileBaseName + Number + extension
     '''
+    # TODO: Switch to a exponential+binary search (log(n) time as opposed to current O(n))?
+        # Even better if we only interact with the file system a single time (getting list of files/folders)
     fileNumber = 0
     filePath = None
     while filePath == None or os.path.exists(filePath):
@@ -101,23 +106,39 @@ def findNextAvailableNumberedFileName(fileBaseName="monteCarloLog", extension=".
 
     return filePath
 
+@lru_cache(maxsize=1)
+def inRepository():
+    currentDirectory = os.path.dirname(__file__)
+    if ".git" in os.listdir(currentDirectory):
+        return True
+    
+    return False
+
+@lru_cache(maxsize=1)
 def getSystemInfo(printToConsole=False):
     ''' Returns string array containing info about github status, machine type, date, etc... '''
     
     from datetime import datetime
-    from subprocess import check_output
+    from subprocess import check_output, CalledProcessError
     from platform import platform
     from os import getlogin
     
     result = []
 
-    try:
-        # Add current git status
-        currentCommit = check_output(['git', 'rev-parse', 'HEAD']).decode()[:-1]
-        currentBranch = check_output(['git', 'rev-parse', '--abbrev-ref', 'HEAD']).decode()[:-1]
-        result.append("# RocketSimulator, branch: {}, latest commit: {}".format(currentBranch, currentCommit))
-    except:
-        result.append("# ERROR: Could not obtain current branch/commit info from git. Ensure command-line version of git is installed: https://git-scm.com/downloads")
+    # Add version number
+    result.append("# MAPLEAF v{}".format(MAPLEAF.__version__))
+    
+    # Check whether running in a repository (installed from source) or not (installed from pypi)
+    if inRepository():
+        try:
+            # Add current git status
+            currentDirectory = os.path.dirname(__file__)
+            currentCommit = check_output(['git', 'rev-parse', 'HEAD'], cwd=currentDirectory).decode()[:-1]
+            currentBranch = check_output(['git', 'rev-parse', '--abbrev-ref', 'HEAD'], cwd=currentDirectory).decode()[:-1]
+            result.append("# MAPLEAF, branch: {}, latest commit: {}".format(currentBranch, currentCommit))
+
+        except CalledProcessError:
+            result.append("# ERROR: Could not obtain current branch/commit info from git. Ensure command-line version of git is installed: https://git-scm.com/downloads")
     
     # Add date/time
     now = datetime.now().strftime("%d/%m/%Y %H:%M:%S")
@@ -154,7 +175,7 @@ def getSimDefinitionAndDefaultValueDictsForOutput(simDefinition, printToConsole=
     
     # Add default value dict
     from pprint import pformat
-    from MAPLEAF.IO.SimDefinition import defaultConfigValues
+    from MAPLEAF.IO import defaultConfigValues
     
     stringResultArray.append("\n---- Start Default Value Dictionary ----\n")
     stringResultArray.append(pformat(defaultConfigValues))
@@ -187,10 +208,7 @@ def postProcessForceEvalLog(logFilePath, refArea=1, refLength=1):
     print("Post Processing (Calculating aerodynamic coefficients) log file: {}".format(logFilePath))
 
     # Load log file into a dataframe
-    with open(logFilePath, 'r') as file:
-        lines = file.readlines()
-    skipRows, skipFooter = Plotting._getSkipRowsAndFooters_forceEvalLog(lines)
-    forcesLog = pd.read_csv(logFilePath, sep="\s+", skiprows=skipRows, dtype=np.float64, skipfooter=skipFooter, engine='python')
+    forcesLog = pd.read_csv(logFilePath, dtype=np.float64)
 
     # Check if the file is already post-processed
     if "AeroCFX(N)" in forcesLog.columns:
@@ -213,8 +231,9 @@ def postProcessForceEvalLog(logFilePath, refArea=1, refLength=1):
 
     # Compute force coefficients
     for col in forcesLog.columns:
-        isForcesColumn = "FX(N)" in col or "FY(N)" in col or "FZ(N)" in col
-        isMomentColumn = "MX(Nm)" in col or "MY(Nm)" in col or "MZ(Nm)" in col
+        isForcesColumn = "(N)" in col
+        isMomentColumn = "(Nm)" in col
+
         if isForcesColumn:
             # Avoid simply replacing 'F' with 'CF' in case names contain 'F' elsewhere
             coordDir = col[-4]
@@ -251,10 +270,9 @@ def postProcessForceEvalLog(logFilePath, refArea=1, refLength=1):
     forcesLog.drop(columns=columnsToDrop)
 
     # Write expanded log to file
-    newLogFilePath = logFilePath.replace(".txt", "_expanded.txt")
-    forcesLog.to_csv(newLogFilePath, sep='\t', index=False)
+    newLogFilePath = logFilePath.replace(".csv", "_expanded.csv")
+    forcesLog.to_csv(newLogFilePath, index=False)
 
     print("Writing expanded level 3 log to: {}".format(newLogFilePath))
 
     return newLogFilePath
-
