@@ -17,10 +17,10 @@ import mpl_toolkits.mplot3d.axes3d as p3
 import numpy as np
 import pandas as pd
 
-from MAPLEAF.Motion.Interpolation import interpolateRigidBodyStates, linInterpWeights
-from MAPLEAF.IO.RocketFlight import RocketFlight
-from MAPLEAF.Motion.CythonQuaternion import Quaternion
-from MAPLEAF.Motion.CythonVector import Vector
+from MAPLEAF.Motion import interpolateRigidBodyStates, linInterpWeights
+from MAPLEAF.IO import RocketFlight, getAbsoluteFilePath
+from MAPLEAF.Motion import Quaternion
+from MAPLEAF.Motion import Vector
 
 # For all plots for publications: (match publication font)
 plt.rcParams["font.family"] = "Times New Roman"
@@ -29,9 +29,8 @@ plt.rcParams["font.weight"] = "normal"
 
 logFileCache = OrderedDict()
 ''' Will keep the contents of the last n log files loaded cached, to avoid re-loading repeatedly when making lots of plots '''
-
-numberOfFilesToKeepCached = 10
-''' Number of log files to keep cached '''
+maxLogFileCacheSize = 10
+''' Max number of log files to keep cached simultaneously '''
 
 ### Plot data from log files ###
 def plotFromLogFiles(logFilePaths, plotDefinitionString, showPlot=True):
@@ -103,13 +102,16 @@ def tryPlottingFromLog(logPath, columnSpecs, columnsToExclude=[], ax=None, showP
     
     return names
 
-def getLoggedColumns(logPath, columnSpecs, columnsToExclude=[], sep="\s+"):
+def getLoggedColumns(logPath, columnSpecs, columnsToExclude=[], sep="\s+", enableCache=True):
     '''
+        Obtains columns matching one or more regex expressions in a log file.
+        By default, log file contents are cached by file name.
+
         Inputs:
             logPath:            (string) path to a simulationLog or forceEvaluationLog file
             columnSpecs:        (list (string)) list of partial/full column names and/or regex expressions, to identify the desired columns
             columnsToExcluse:   (list (string)) list of full column names to exclude
-            sep:                (string) controls the separator pandas.read_csv uses to load the file ('\s+' / whitespace) for rocketSimulator's log files. "," for .csv
+            sep:                (string) controls the separator pandas.read_csv uses to load the file ('\s+' / whitespace) for MAPLEAF's log files. "," for .csv
 
         Outputs:
             Returns: matchingColumnData (list (list (float))), matchingColumnNames (list (string))
@@ -118,26 +120,31 @@ def getLoggedColumns(logPath, columnSpecs, columnsToExclude=[], sep="\s+"):
     if isinstance(columnSpecs, str):
         columnSpecs = [ columnSpecs ]
     
-    if logPath in logFileCache:
+    if logPath in logFileCache and enableCache:
         df = logFileCache[logPath]
     else:
-        with open(logPath, 'r') as file:
-            lines = file.readlines()
-
-        #### Figure out which lines are not part of the data table ####
-        if "simulationLog" in logPath:
-            skipRows, skipFooter = _getSkipRowsAndFooters_mainSimLog(lines)
-        elif "forceEvaluationLog" in logPath:
-            skipRows, skipFooter = _getSkipRowsAndFooters_forceEvalLog(lines)
+        if ".csv" in logPath:
+            df = pd.read_csv(logPath, dtype=np.float64)
+        
         else:
-            skipRows, skipFooter = (0, 0)
+            # LEGACY TODO: Remove
+            with open(logPath, 'r') as file:
+                lines = file.readlines()
 
-        #### Read file, grab desired columns ####
-        # Read file, skipping any lines found in the previous section
-        df = pd.read_csv(logPath, sep=sep, skiprows=skipRows, dtype=np.float64, skipfooter=skipFooter, engine='python')
+            #### Figure out which lines are not part of the data table ####
+            if "simulationLog" in logPath:
+                skipRows, skipFooter = _getSkipRowsAndFooters_mainSimLog(lines)
+            elif "forceEvaluationLog" in logPath:
+                skipRows, skipFooter = _getSkipRowsAndFooters_forceEvalLog(lines)
+            else:
+                skipRows, skipFooter = (0, 0)
+
+            #### Read file, grab desired columns ####
+            # Read file, skipping any lines found in the previous section
+            df = pd.read_csv(logPath, sep=sep, skiprows=skipRows, dtype=np.float64, skipfooter=skipFooter, engine='python')
             
         logFileCache[logPath] = df
-        if len(logFileCache) > numberOfFilesToKeepCached:
+        if len(logFileCache) > maxLogFileCacheSize:
             logFileCache.popitem(last=False)
     
     matchingColumnData = []
@@ -265,7 +272,7 @@ def plot_Earth_Mayavi(earthTexture='MAPLEAF/IO/blue_marble_spherical_splitFlippe
 
     # load and map the texture
     img = tvtk.JPEGReader()
-    img.file_name = earthTexture
+    img.file_name = getAbsoluteFilePath(earthTexture)
     texture = tvtk.Texture(input_connection=img.output_port, interpolate=1)
     # (interpolate for a less raster appearance when zoomed in)
 
@@ -369,10 +376,11 @@ def _keepNTimeSteps(flights, nFramesToKeep=600):
     else:
         return flights
 
-def _get3DPlotSize(Positions, sizeMultiple=1.1):
+def _get3DPlotSize(flight, sizeMultiple=1.1):
     '''
         Finds max X, Y, or Z distance from the origin reached during the a flight. Used to set the 3D plot size (which will be equal in all dimensions)
     '''
+    Positions = flight.Positions
     centerOfPlot = Vector(mean(Positions[0]), mean(Positions[1]), mean(Positions[2]))
 
     xRange = max(Positions[0]) - min(Positions[0])
@@ -630,7 +638,7 @@ def flightAnimation(flights, showPlot=True, saveAnimFileName=None):
         flight.Positions = Positions
 
     # Set xyz size of plot - equal in all dimensions
-    axisDimensions, centerOfPlot = _get3DPlotSize(flights[0].Positions) # Assumes the top stage travels the furthest
+    axisDimensions, centerOfPlot = _get3DPlotSize(flights[0]) # Assumes the top stage travels the furthest
 
     # Calculate frames at which engine turns off and main chute deploys
     for flight in flights:
@@ -638,6 +646,7 @@ def flightAnimation(flights, showPlot=True, saveAnimFileName=None):
         flight.mainChuteTimeStep = _findEventTimeStepNumber(flight, flight.mainChuteDeployTime)
 
     if flights[0].actuatorDefls != None:
+        # Assume all actuated systems are canards #TODO: This needs updating
         # Assuming canards always on the top stage
         nCanards = len(flights[0].actuatorDefls)
     else:
@@ -740,3 +749,22 @@ def plotAndSummarizeScalarResult(scalarList, name="Apogee", monteCarloLogger=Non
 
     if showPlot:
         plt.show()
+
+### Colors ###
+def getColorPalette(nColors=4):
+    ''' Returns a list of nColors colors '''
+    if nColors > 8:
+        raise ValueError("Too many colors requested: {} (Max 8)".format(nColors))
+    if nColors < 3:
+        raise ValueError("Too few colors requested: {} (Min 3)".format(nColors))
+
+    colorPaletteDict = {        
+        3: [ '#003f5c', '#bc5090', '#ffa600'],
+        4: [ '#003f5c', '#7a5195', '#ef5675', '#ffa600'],
+        5: [ '#003f5c', '#58508d', '#bc5090', '#ff6361','#ffa600'],
+        6: [ '#003f5c', '#444e86', '#955196', '#dd5182', '#ff6e54', '#ffa600'],
+        7: [ '#003f5c', '#374c80', '#7a5195', '#bc5090', '#ef5675', '#ff764a', '#ffa600'],
+        8: [ '#003f5c', '#2f4b7c', '#665191', '#a05195', '#d45087', '#f95d6a', '#ff7c43', '#ffa600' ]
+    }
+
+    return colorPaletteDict[nColors]
\ No newline at end of file
