@@ -6,200 +6,11 @@ Used throughout the aeordynamics functions
 '''
 
 import math
-from functools import wraps
 
-from MAPLEAF.Motion.CythonVector import Vector
+from MAPLEAF.Motion import (AeroParameters, ForceMomentSystem, Quaternion,
+                            Vector)
+from MAPLEAF.Utilities import cacheLastResult
 
-#### Function decorators ####
-def cacheLastResult(func):
-    '''
-        Function decorator that caches that function's last return value.
-            Could use @functools.lru_cache(maxsize=1) to achieve the same behavior, but probably slower
-    '''
-    cache = dict()
-    cache[1] = [] # Cache last argument list here
-    cache[2] = None # Cache last result here
-
-    def memoized_func(*args):
-        # Return cached result if available
-        if cache[1] == args:
-            return cache[2]
-        
-        # Compute and cache result
-        result = func(*args)
-        cache[1] = args
-        cache[2] = result
-        return result
-
-    return memoized_func
-
-def logForceResult(func):
-    def loggedAeroFunction(*args):
-        aeroForce = func(*args)
-        
-        # Log to force evaluation log
-        rocketComponent = args[0]
-        rocketComponent.rocket.appendToForceLogLine(" {:>10.4f} {:>10.4f}".format(aeroForce.force, aeroForce.moment))
-        
-        return aeroForce
-    return loggedAeroFunction
-
-
-
-#### Functions with standardized interfaces - used for interpolating ####
-def getAltitude(state, environment, _=None):
-    return environment.ASLAltitude
-
-@cacheLastResult
-def getMachNumber(state, environment, _=None):
-    # Otherwise compute and cache the result
-    airVel = state.velocity - environment.Wind
-    Ma = airVel.length() / math.sqrt(1.4 * 287 * environment.Temp)    #TODO: Get gamma and R from default dict / config file
-    return Ma
-
-@cacheLastResult
-def getReynoldsNumber(state, environment, length):
-    unitRe = getLocalFrameAirVel(state, environment).length() * environment.Density / environment.DynamicViscosity
-    return unitRe * length
-
-@cacheLastResult
-def getTotalAOA(state, environment, _=None):
-    ''' Returns the **magnitude** of the angle between the local-frame wind vector and longitudinal axis, in degrees '''
-    localFrameAirVel = getLocalFrameAirVel(state, environment)
-
-    if localFrameAirVel.length() == 0:
-        AOA = 0
-    else:
-        AOA = localFrameAirVel.angle(Vector(0,0,-1))
-
-    return AOA
-
-def getAOA(state, environment, _=None):
-    ''' 
-        Returns angle of attack (between the local Z-X plane and the airspeed vector)
-        Measures rotations about the local +y axis (compared to wind-aligned state)
-    '''
-    localFrameAirVel = getLocalFrameAirVel(state, environment)
-
-    if localFrameAirVel.length() == 0:
-        return 0
-
-    # Zero out the Y-component (Sideslip component)
-    localFrameAirVel.Y = 0
-    
-    # Compute angle with longitudinal axis
-    if localFrameAirVel.X > 0:
-        return localFrameAirVel.angle(Vector(0,0,-1))
-    else:
-        return -localFrameAirVel.angle(Vector(0,0,-1))
-
-def getAOSS(state, environment, _=None):
-    ''' 
-        Returns angle of sideslip (between the local Z-Y plane and the airspeed vector)
-        Measures rotations about the local +z axis (compared to wind-aligned state)
-    '''
-    localFrameAirVel = getLocalFrameAirVel(state, environment)
-
-    if localFrameAirVel.length() == 0:
-        return 0
-
-    # Zero out the X-component (AOA component)
-    localFrameAirVel.X = 0
-
-    # Compute angle with longitudinal axis
-    if localFrameAirVel.Y < 0:
-        return localFrameAirVel.angle(Vector(0,0,-1))
-    else:
-        return -localFrameAirVel.angle(Vector(0,0,-1))
-
-@cacheLastResult
-def getRollAngle(state, environment, _=None):
-    ''' 
-        Returns the angular distance between axis about which the rocket rotates to create an angle of attack, and the X-axis, in degrees
-        If the normal force direction is aligned with the y-axis, roll angle = 0
-        If the normal force direction is aligned with the x-axis, roll angle = 90 degrees
-    '''
-
-    # TODO: Convert this to using radians
-    normalAeroForceDirection = getNormalAeroForceDirection(state, environment)
-    y = normalAeroForceDirection*(-1)
-    if(y.length() < 0.01): #Check for when there is a zero angle of attack (Numerical issue)
-        rollAngle = 0
-    else:
-        rollAngle = math.degrees(math.atan2(-y.Y, y.X))
-        rollAngle = (-1)*rollAngle
-        rollAngle %= 360
-
-    return rollAngle
-
-def getAeroPropertiesList(aeroFunctionList, state, environment):
-    '''
-        Returns a vector of parameters specified by strings - used for interpolating by user-specified parameters
-
-        aeroFunctionVector:     vector of references to standardized-interface functions below
-        state:                  rocket rigid body state
-        environment:            environmentalConditions named tuple
-    '''
-    aeroProperties = []
-    for fn in aeroFunctionList:
-        aeroProperties.append(fn(state, environment, 1))
-
-    return aeroProperties
-
-stringToAeroFunctionMap = {
-    # This defines the relationship between strings obtained from sim config files,
-        # To functions that obtain some parameter, often used for interpolation of tabulated properties
-        # Use by control systems, TabulatedAeroForce, and TabulatedInertia objects
-    "Mach":         getMachNumber,
-    "Altitude":     getAltitude,
-    "UnitReynolds": getReynoldsNumber,
-    "TotalAOA":     getTotalAOA,
-    "RollAngle":    getRollAngle,
-    "AOA":          getAOA,
-    "AOSS":         getAOSS
-}
-
-
-
-#### Other Functions ####
-@cacheLastResult
-def getLocalFrameAirVel(state, environment):
-    ''' Returns the vector representing the motion of freestream air relative to the rocket, in the rocket frame '''
-    return state.orientation.conjugate().rotate(state.velocity - environment.Wind)*-1
-
-def getAirVelRelativeToVehicle(state, environment):
-    ''' 3DoF version of getLocalFrameAirVel '''
-    return (state.velocity - environment.Wind) * -1
-
-@cacheLastResult
-def getNormalAeroForceDirection(state, environment):
-    localFrameAirVel = getLocalFrameAirVel(state, environment)
-    if localFrameAirVel[0] == 0.0 and localFrameAirVel[1] == 0.0:
-        # Return random vector if AOA == 0 - force will be zero anyhow
-        return Vector(1,0,0)
-    else:
-        return Vector(localFrameAirVel[0], localFrameAirVel[1], 0).normalize()
-
-@cacheLastResult
-def getDynamicPressure(state, environment):
-    try:
-        velMag = getLocalFrameAirVel(state, environment).length()
-    except AttributeError:
-        #3DoF mode
-        velMag = getAirVelRelativeToVehicle(state, environment).length()
-
-    return velMag*velMag * environment.Density / 2
-
-@cacheLastResult
-def getBeta(Mach):
-    #Define the compressibility factor depending on if we are subsonic or supersonic, if we are at M = 1 we set
-    #the compressibility factor to a hardcoded literal close to zero (equations break if not)
-    if Mach <= 0.9: 
-        return math.sqrt(1 - Mach**2) #Compressibility correction factor for subsonic flow
-    elif Mach >= 1.1:
-        return math.sqrt(Mach**2 - 1) #For supersonic flow
-    else:
-        raise NotImplementedError("Beta not valid in the range 0.9 < M < 1.1 - goes to infinity")
 
 def _getCPZ(componentForces) -> int:
     normalForce = Vector(componentForces.force.X, componentForces.force.Y, 0)
@@ -217,7 +28,6 @@ def _getCPZ(componentForces) -> int:
     return componentForces.location.Z - momentComponent/normalForceMagnitude
 
 
-
 #### Constants ####
 def getGamma():
     return 1.4
@@ -275,7 +85,7 @@ def getSkinFrictionCoefficient(state, environment, length, Mach, surfaceRoughnes
         Outputs:
           * Returns the skin friction coefficient normalized by **wetted area**, must be converted to be normalized by cross-sectional area before summing with other coefficients.
     '''
-    Re = getReynoldsNumber(state, environment, length)
+    Re = AeroParameters.getReynoldsNumber(state, environment, length)
     smoothSurface = (surfaceRoughness == 0) # True or False
 
     if smoothSurface:
@@ -338,7 +148,7 @@ def getCylindricalSkinFrictionDragCoefficientAndRollDampingMoment(state, environ
     avgRadius = (wettedArea / length) / (2*math.pi)
     rollingSurfaceVel = avgRadius * state.angularVelocity.Z
     # Assume velocities are perpendicular
-    airVel = getLocalFrameAirVel(state, environment).length()
+    airVel = AeroParameters.getLocalFrameAirVel(state, environment).length()
     # Use total velocity to redimensionalize coefficient
     totalVelSquared = airVel**2 + rollingSurfaceVel**2
 
@@ -393,7 +203,7 @@ def getBluntBodyCd_ZeroBaseDrag(Mach):
 @cacheLastResult
 def getDragToAxialForceFactor(AOA):
     ''' Expected input is radians angle of attack - converts dragCoefficient to axialForceCoefficient: openRocket documentation pg. 53 '''
-    # Angle of attack will not be negative if obtained from getTotalAOA above
+    # Angle of attack will not be negative if obtained from getTotalAOA in AeroParameters
         # But can be negative when fins are calculating their angles of attack, hence the abs()
     AOA = math.degrees(abs(AOA))
 
@@ -404,6 +214,60 @@ def getDragToAxialForceFactor(AOA):
         return 0.000006684*AOA**3 - 0.0010727204*AOA**2 + 0.030677323*AOA + 1.055660807
 
 
+#### Creating ForceMomentSystems from Coefficients ####
+def forceFromCdCN(rocketState, environment, Cd, CN, CPLocation, refArea, moment=None):
+    ''' 
+        Convenience function for Barrowman Aero methods
+        Initialize ForceMomentSystem from aerodynamic coefficients Cd and CN
+        Cd should NOT already be adjusted for angle of attack 
+        Moment should be a dimensional moment vector
+    '''
+    angleOfAttack = AeroParameters.getTotalAOA(rocketState, environment)
+    q = AeroParameters.getDynamicPressure(rocketState, environment)
+
+    #### Axial Force ####
+    CA = getDragToAxialForceFactor(angleOfAttack) * Cd
+    axialForce = Vector(0, 0, -CA) #Axial force is in the negative Z-direction 
+
+    #### Normal Force ####
+    normalForce = AeroParameters.getNormalAeroForceDirection(rocketState, environment) * CN
+
+    totalForce = (axialForce + normalForce) * refArea * q
+    return ForceMomentSystem(totalForce, CPLocation, moment)
+
+def forceFromCoefficients(rocketState, environment, Cd, Cl, CMx, CMy, CMz, CPLocation, refArea, refLength):
+    ''' Initialize ForceMomentSystem from all aerodynamic coefficients '''
+    q = AeroParameters.getDynamicPressure(rocketState, environment)
+    if q == 0.0:
+        # No force without dynamic pressure
+        return ForceMomentSystem(Vector(0,0,0))
+    nonDimConstant = q * refArea
+
+    #### Drag ####
+    localFrameAirVel = AeroParameters.getLocalFrameAirVel(rocketState, environment)
+    dragDirection = localFrameAirVel.normalize()
+    dragForce = dragDirection * Cd
+
+    #### Lift ####
+    # Find the lift force direction
+        # Lift force will be in the same plane as the normalForceDirection & rocketFrameAirVel vectors
+        # But, the lift force will be perpendicular to rocketFraeAirVel, whereas the normalForceDirection might form an acute angle with it
+        # Rotate the normal force direction vector until it's perpendicular with rocketFrameAirVel
+    normalForceDir = AeroParameters.getNormalAeroForceDirection(rocketState, environment)
+    rotAxis = localFrameAirVel.crossProduct(normalForceDir)
+    angle = math.pi/2 - localFrameAirVel.angle(normalForceDir)
+    rotatorQuat = Quaternion(rotAxis, angle)
+    liftDirection = rotatorQuat.rotate(normalForceDir)
+    liftForce = liftDirection * Cl
+
+    # Combine and redimensionalize
+    totalForce = (dragForce + liftForce) * nonDimConstant
+
+    #### Moments ####
+    moments = Vector(CMx, CMy, CMz) * nonDimConstant * refLength
+
+    return ForceMomentSystem(totalForce, CPLocation, moments)
+
 
 #### Barrowman ####
 def Barrowman_GetCPLocation(length, crossSectionalArea_top, crossSectionalArea_bottom, volume, topLocation=Vector(0,0,0)):
