@@ -2,29 +2,28 @@
 In charge of initializing rocket components. Add new components to `stringNameToClassMap` to make them instantiate themselves when included in a Rocket
 '''
 
-from MAPLEAF.IO.SubDictReader import SubDictReader
-from MAPLEAF.Rocket.BoatTail import BoatTail, Transition
-from MAPLEAF.Rocket.Bodytube import Bodytube
-from MAPLEAF.Rocket.Fins import FinSet
-from MAPLEAF.Rocket.Motor import Motor
-from MAPLEAF.Rocket.Nosecone import Nosecone
-from MAPLEAF.Rocket.RecoverySystem import RecoverySystem
-from MAPLEAF.Rocket.RocketComponents import (AeroDamping, AeroForce, FixedForce,
-                                         FixedMass, FractionalJetDamping,
-                                         TabulatedAeroForce, TabulatedInertia)
-    
+from MAPLEAF.IO import SubDictReader
+from MAPLEAF.Motion import Vector
+from MAPLEAF.Rocket import (
+    AeroDamping, AeroForce, BoatTail, BodyTube, FinSet, FixedForce, FixedMass,
+    FractionalJetDamping, NoseCone, RecoverySystem, SampleStatefulComponent, TabulatedAeroForce,
+    TabulatedInertia, TabulatedMotor, Transition)
+
+__all__ = [ "stringNameToClassMap", "rocketComponentFactory", "initializeForceLogging" ]
+
 stringNameToClassMap = {
     "AeroDamping":          AeroDamping,
     "AeroForce":            AeroForce,
     "BoatTail":             BoatTail,
-    "Bodytube":             Bodytube,
+    "Bodytube":             BodyTube,
     "FinSet":               FinSet,
     "Force":                FixedForce,
     "FractionalJetDamping": FractionalJetDamping,
     "Mass":                 FixedMass,
-    "Motor":                Motor,
-    "Nosecone":             Nosecone,
+    "Motor":                TabulatedMotor,
+    "Nosecone":             NoseCone,
     "RecoverySystem":       RecoverySystem,
+    "StatefulSample":       SampleStatefulComponent,
     "TabulatedAeroForce":   TabulatedAeroForce,
     "TabulatedInertia":     TabulatedInertia,
     "Transition":           Transition,
@@ -46,5 +45,17 @@ def rocketComponentFactory(subDictPath, rocket, stage):
     className = componentDictReader.getString("class")
     referencedClass = stringNameToClassMap[className]
     
-    # Initialize it
-    return referencedClass(componentDictReader, rocket, stage)
+    # Initialize the rocket component
+    newComponent = referencedClass(componentDictReader, rocket, stage)
+
+    # Initialize logging component forces (if desired)
+    initializeForceLogging(newComponent, subDictPath, rocket)
+
+    return newComponent
+
+def initializeForceLogging(component, subDictPath, rocket):
+    if rocket.derivativeEvaluationLog is not None:
+        componentName = subDictPath[subDictPath.index(".")+1:]
+        zeroVector = Vector(0,0,0)
+        component.forcesLog = rocket.derivativeEvaluationLog.addColumn(componentName + "F(N)", zeroVector)
+        component.momentsLog = rocket.derivativeEvaluationLog.addColumn(componentName + "M(Nm)", zeroVector)
\ No newline at end of file
