@@ -9,38 +9,46 @@ from abc import ABC, abstractmethod
 from typing import List, Union
 
 import numpy as np
-from scipy.interpolate import LinearNDInterpolator
+from MAPLEAF.Motion import (AeroParameters, ForceMomentSystem, Inertia,
+                            NoNaNLinearNDInterpolator, RigidBodyState, Vector,
+                            linInterp)
 
-import MAPLEAF.Rocket.AeroFunctions as AeroFunctions
-from MAPLEAF.ENV.Environment import EnvironmentalConditions
-from MAPLEAF.Motion.Interpolation import linInterp
-from MAPLEAF.IO.SubDictReader import SubDictReader
-from MAPLEAF.Motion.CythonVector import Vector
-from MAPLEAF.Motion.ForceMomentSystem import ForceMomentSystem
-from MAPLEAF.Motion.Inertia import Inertia
-from MAPLEAF.Motion.RigidBodyStates import RigidBodyState, RigidBodyState_3DoF
-from MAPLEAF.Rocket.AeroFunctions import logForceResult
+from . import AeroFunctions
 
+__all__ = [ "RocketComponent", "BodyComponent", "PlanarInterface", "FixedMass", "FixedForce", "AeroForce", "AeroDamping", "TabulatedAeroForce", "TabulatedInertia", "FractionalJetDamping" ]
 
 class RocketComponent(ABC):
     ''' Interface definition for rocket components '''
     @abstractmethod
-    def __init__(self, componentDictReader: SubDictReader, rocket, stage):
+    def __init__(self, componentDictReader, rocket, stage):
         return
 
     @abstractmethod
-    def getInertia(self, time: float, state: Union[RigidBodyState, RigidBodyState_3DoF]) -> Inertia:
+    def getInertia(self, time, state) -> Inertia:
         return
 
     @abstractmethod
-    def getAeroForce(self, rocketState: Union[RigidBodyState, RigidBodyState_3DoF], time: float, environmentalConditions: EnvironmentalConditions, rocketCG: Vector) -> ForceMomentSystem:
+    def getAppliedForce(self, rocketState, time, environmentalConditions, rocketCG) -> ForceMomentSystem:
         return
 
+    #### Other optional functions ####
+    # def getExtraParametersToIntegrate(self):
+    '''
+        Override this method to make MAPLEAF integrate additional parameters (in addition to rigid body motion)
+        Should return three lists
+            - A list of (string) parameter names
+                - The parameter will then be accessible under state.parameterName whenever the rocket state is available
+            - A list of intial parameter values
+                - Values can be scalars or vectors, or any other parameter that implements addition, subtraction and multiplication/division by scalars
+            - A list of parameter derivative functions
+                - Derivative functions should accept two parameters, the current time and the current state object and return the corresponding derivative of the value of that parameter
+    '''
+
 class BodyComponent(ABC):
     ''' 
         Class that defines interface for axisymmetric body components.
         Contains logic for detecting adjacent body components & defining interfaces with them 
-        Examples: `MAPLEAF.Rocket.Nosecone.Nosecone`, `MAPLEAF.Rocket.Stage.Stage`
+        Examples: `MAPLEAF.Rocket.NoseCone`, `MAPLEAF.Rocket.Stage`
     '''
     # Override these attributes in child classes to change whether they can connect to components above/below them
     canConnectToComponentAbove = True
@@ -61,9 +69,6 @@ class BodyComponent(ABC):
         else:
             return None
 
-    def getLogHeader(self):
-        return " {}FX(N) {}FY(N) {}FZ(N) {}MX(Nm) {}MY(Nm) {}MZ(Nm)".format(*[self.name]*6)
-
     def _getCenterOfPressure(self, *args) -> Vector:
         return self.CPLocation
 
@@ -123,7 +128,7 @@ class PlanarInterface():
         return componentInterfaces
 
     @classmethod
-    def sortByZLocation(cls, components) -> List[RocketComponent]:
+    def sortByZLocation(cls, components, state) -> List[RocketComponent]:
         ''' 
             Sort the components in order from top to bottom, component.position.Z
             This function could be relocated somewhere more suitable, at the time of writing, it is only being used to order components before creating interfaces b/w them
@@ -132,8 +137,7 @@ class PlanarInterface():
             try:
                 return component.position.Z
             except AttributeError:
-                zeroState = RigidBodyState()
-                return component.getInertia(0, zeroState).CG.Z
+                return component.getInertia(0, state).CG.Z
 
         components.sort(key=getZPosition, reverse=True)
         return components
@@ -173,7 +177,7 @@ class FixedMass(RocketComponent):
     def getCG(self, time):
         return self.inertia.CG
 
-    def getAeroForce(self, rocketState, time, environment, CG):
+    def getAppliedForce(self, rocketState, time, environment, CG):
         return self.zeroForce
 
 class FixedForce(RocketComponent):
@@ -196,13 +200,9 @@ class FixedForce(RocketComponent):
     def getInertia(self, time, state):
         return self.inertia
 
-    @logForceResult
-    def getAeroForce(self, rocketState, time, environment, rocketCG):
+    def getAppliedForce(self, rocketState, time, environment, rocketCG):
         return self.force
 
-    def getLogHeader(self):
-        return " {}FX(N) {}FY(N) {}FZ(N) {}MX(Nm) {}MY(Nm) {}MZ(Nm)".format(*[self.name]*6)
-
 class AeroForce(RocketComponent):
     ''' A zero-Inertia component with constant aerodynamic coefficients '''
     # Object is just a force, inertia is zero
@@ -227,12 +227,8 @@ class AeroForce(RocketComponent):
     def getInertia(self, time, state):
         return self.inertia
 
-    @logForceResult
-    def getAeroForce(self, state, time, environment, rocketCG):
-        return ForceMomentSystem.fromAllCoefficients(state, environment, *self.aeroCoeffs, self.position, self.Aref, self.Lref)
-
-    def getLogHeader(self):
-        return " {}FX(N) {}FY(N) {}FZ(N) {}MX(Nm) {}MY(Nm) {}MZ(Nm)".format(*[self.name]*6)
+    def getAppliedForce(self, state, time, environment, rocketCG):
+        return AeroFunctions.forceFromCoefficients(state, environment, *self.aeroCoeffs, self.position, self.Aref, self.Lref)
 
 class AeroDamping(AeroForce):
     ''' A zero-inertia component with constant aerodynamic damping coefficients '''
@@ -252,9 +248,8 @@ class AeroDamping(AeroForce):
         self.yDampingCoeffs = componentDictReader.getVector("yDampingCoeffs")
         self.xDampingCoeffs = componentDictReader.getVector("xDampingCoeffs")
     
-    @logForceResult
-    def getAeroForce(self, state, time, environment, rocketCG):
-        airspeed = max(AeroFunctions.getLocalFrameAirVel(state, environment).length(), 0.0000001)
+    def getAppliedForce(self, state, time, environment, rocketCG):
+        airspeed = max(AeroParameters.getLocalFrameAirVel(state, environment).length(), 0.0000001)
         redimConst = self.Lref / (2*airspeed)
         # Calculate moment coefficients from damping coefficients
         localFrameAngularVelocity = Vector(*state.angularVelocity)
@@ -263,7 +258,7 @@ class AeroDamping(AeroForce):
         xMomentCoeff = self.xDampingCoeffs * localFrameAngularVelocity * redimConst
         momentCoeffs = [ xMomentCoeff, yMomentCoeff, zMomentCoeff ]
 
-        return ForceMomentSystem.fromAllCoefficients(state, environment, 0, 0, *momentCoeffs, self.position, self.Aref, self.Lref)
+        return AeroFunctions.forceFromCoefficients(state, environment, 0, 0, *momentCoeffs, self.position, self.Aref, self.Lref)
 
 class TabulatedAeroForce(AeroForce):
     ''' A zero-inertia component with aerodynamic coefficients that are tabulated according to one or more parameters (ex. AOA) '''
@@ -292,14 +287,14 @@ class TabulatedAeroForce(AeroForce):
         i = 0
         while i < len(columnNames):
             col = columnNames[i]
-            if col in AeroFunctions.stringToAeroFunctionMap:
-                self.parameterFunctions.append(AeroFunctions.stringToAeroFunctionMap[col])
+            if col in AeroParameters.stringToAeroFunctionMap:
+                self.parameterFunctions.append(AeroParameters.stringToAeroFunctionMap[col])
             else:
                 break
             i += 1
 
         # Continue parsing column names - aero coefficient names now            
-        # This is the ordering expected by ForceMomentSystem.fromAllCoefficients
+        # This is the ordering expected by AeroFunctions.forceFromCoefficients
         aeroCoeffStrings = [ "CD", "CL", "CMx", "CMy", "CMz" ]
         self.aeroCoeffIndices = [] # Provides mapping between value column position in interpolation table & position in output aero coefficient list (ordered like aeroCoeffStrings above)
         while i < len(columnNames):
@@ -310,7 +305,7 @@ class TabulatedAeroForce(AeroForce):
                 
             else:
                 raise ValueError("ERROR: One of the following columns: {} did not match any of the expected columns names: Keys: {}, values: {}. \
-                    Or was in the wrong order. All key columns must come BEFORE value columns.".format(columnNames, AeroFunctions.stringToAeroFunctionMap.keys(), aeroCoeffStrings))
+                    Or was in the wrong order. All key columns must come BEFORE value columns.".format(columnNames, AeroParameters.stringToAeroFunctionMap.keys(), aeroCoeffStrings))
             i += 1
 
         # Load the data table to be interpolated
@@ -322,14 +317,14 @@ class TabulatedAeroForce(AeroForce):
 
         if nKeyCols > 1:
             # Create n-dimensional interpolation function for aero coefficients
-            self._interpAeroCoefficients = LinearNDInterpolator(keys, aeroCoefficients)
+            self._interpAeroCoefficients =  NoNaNLinearNDInterpolator(keys, aeroCoefficients, filePath)
         else:
-            # Save to use with MAPLEAF.Motion.Interpolation.linInterp
+            # Save to use with MAPLEAF.Motion.linInterp
             self.keys = [ key[0] for key in keys ]
             self.values = aeroCoefficients
 
     def _getAeroCoefficients(self, state, environment):
-        keys = AeroFunctions.getAeroPropertiesList(self.parameterFunctions, state, environment)
+        keys = AeroParameters.getAeroPropertiesList(self.parameterFunctions, state, environment)
 
         if len(keys) > 1:
             # Multi-dimensional linear interpolation
@@ -345,13 +340,9 @@ class TabulatedAeroForce(AeroForce):
 
         return aeroCoefficients
 
-    @logForceResult
-    def getAeroForce(self, state, time, environment, rocketCG):
+    def getAppliedForce(self, state, time, environment, rocketCG):
         aeroCoefficients = self._getAeroCoefficients(state, environment)
-        return ForceMomentSystem.fromAllCoefficients(state, environment, *aeroCoefficients, self.position, self.Aref, self.Lref)
-
-    def getLogHeader(self):
-        return " {}FX(N) {}FY(N) {}FZ(N) {}MX(Nm) {}MY(Nm) {}MZ(Nm)".format(*[self.name]*6)
+        return AeroFunctions.forceFromCoefficients(state, environment, *aeroCoefficients, self.position, self.Aref, self.Lref)
 
 class TabulatedInertia(RocketComponent):
     ''' A zero-force component with time-varying tabulated inertia '''
@@ -381,7 +372,7 @@ class TabulatedInertia(RocketComponent):
         # MOI is last three columns, CG is the three before that, and mass is column 0
         return Inertia(Vector(*inertiaData[-3:]), Vector(*inertiaData[1:4]), inertiaData[0])
     
-    def getAeroForce(self, rocketState, time, environment, CG):
+    def getAppliedForce(self, rocketState, time, environment, CG):
         return self.zeroForce
 
 class FractionalJetDamping(RocketComponent):
@@ -398,8 +389,7 @@ class FractionalJetDamping(RocketComponent):
         
         self.dampingFraction = componentDictReader.getFloat("fraction")
 
-    @logForceResult
-    def getAeroForce(self, rocketState, time, environmentalConditions, rocketCG):
+    def getAppliedForce(self, rocketState, time, environmentalConditions, rocketCG):
         # Only apply damping force if current stage's engine is firing
             # (Other stage's motors will have different exit planes)
         if time > self.stage.motor.ignitionTime and time < self.stage.engineShutOffTime:
@@ -421,6 +411,3 @@ class FractionalJetDamping(RocketComponent):
 
     def getInertia(self, time, state):
         return self.inertia
-
-    def getLogHeader(self):
-        return " {}FX(N) {}FY(N) {}FZ(N) {}MX(Nm) {}MY(Nm) {}MZ(Nm)".format(*[self.name]*6)
\ No newline at end of file
